# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pyYAHTS']

package_data = \
{'': ['*']}

install_requires = \
['pyats[full]>=22.2,<23.0', 'rich-click>=1.2.1,<2.0.0']

entry_points = \
{'console_scripts': ['pyYAHTS = pyYAHTS.script:run']}

setup_kwargs = {
    'name': 'pyyahts',
    'version': '1.0.1',
    'description': 'An implementation of Cisco pyATS',
    'long_description': '# pyYAHTS\nAn interpretation of pyATS\n\npyYAHTS is a dyanmic, on-demand, YAML-free / testbed.yaml free, implementation of pyATS. \n\nIt works with any IOS / IOS-XE / IOS-XR / NXOS device!\n\nThe results are Rich JSON printed to the screen\n\n## Requirements\npyYAHTS is an extension of pyATS, which is required, and only runs on Linux operating systems\n## Installation\n\n1. Make sure you first install pyATS\n\npip install pyats[full]\n\n2. Install Rich-Click \n\npip install rich-click\n\n3. That\'s it! You are ready to use pyYATS \n\n## Getting Started\n\npyYAHTS works on any Cisco OS IOS / IOS-XE / IOS-XR / NXOS\n\npyYAHTS requires the follow options be speficied at runtime:\n\n1. Hostname of the device - must exactly match the configured hostname\n2. Operating System - Either ios, iosxe, iosxr, or nxos\n3. Username\n4. Password\n5. Either a pyATS Learn Function, such as ospf, or any supported pyATS Parsed CLI Show Command, such as "show ip interface brief"\n\n![Help](images/help01.png)\n\nFor a list of supported Learn Functions please visit [Available Learn Functions](https://pubhub.devnetcloud.com/media/genie-feature-browser/docs/#/models)\n\n![Available Learn Functions](/images/available_learn_functions.png)\n\nFor a list of supported Parsers please visit [Available Show Command Parsers](https://pubhub.devnetcloud.com/media/genie-feature-browser/docs/#/parsers)\n\n![Available Show Parsers](/images/available_show_parsers01.png)\n\nA sample of "Show IP Interface" Parsers for IOS-XE\n![IOS-XE Show IP Interface Sample](/images/available_show_parsers02.png)\n\n\n## Using pyYAHTS\n\nIn any virtual environment with pyATS installed pyYAHTS can be executed several ways\n\n1. Prompted Inputs\n\n```python\n(virtualenv)$ python3 pyYAHTS.py\nHostname: dist-sw01\nOS (ios, iosxe, iosxr, nxos): nxos\nUsername: cisco\nPassword:\nCommand: ospf\n```\n\n2. Directly supplying options\n\n```python\n(virtualenv)$ python3 pyYAHTS.py --hostname dist-sw01 --os nxos --username cisco --password cisco --command ospf\n```\n\n3. Mixing supplied options and prompted responses\n\n```python\n(virtualenv)$ python3 pyYAHTS.py --hostname dist-sw01 --os nxos --username cisco --password cisco\nCommand: ospf\n```\n\n## Creating JSON files\n\nOf course you can also redirect the printed output to a JSON file! \n\n```python\npython3 pyYAHTS.py --hostname dist-sw01 --os nxos --username cisco --password cisco --command ospf > dist_sw01_learned_ospf.json\n```\n## Help\n\npyYAHTS includes a handy Rich Click Help! Simple type:\n\n```python\n$ python3 pyYAHTS.py --help\n```\n\n![More Help](images/help01.png)\n\n## Contact\n\nPlease reach out on Twitter [Twitter](https://twitter.com/john_capobianco) or open an issue if you hit any snags or have any questions!',
    'author': 'John Capobianco',
    'author_email': 'ptcapo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/automateyournetwork/pyYAHTS',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import sys
import rich_click as click
import json
import logging
from pyats.topology import Testbed, Device
from genie import testbed
from rich import print_json

class GetJson():
    def __init__(self, hostname, os, username, password, command):
        self.hostname = hostname
        self.os = os
        self.username = username
        self.password = password
        self.command = command

    def print_json(self):
        print_json(json.dumps(self.capture_state(), indent=4, sort_keys=True))

    # Create Testbed
    def connect_device(self):
        try:
            first_testbed = Testbed('dynamicallyCreatedTestbed')
            testbed_device = Device(self.hostname,
                        alias = self.hostname,
                        type = 'switch',
                        os = self.os,
                        credentials = {
                            'default': {
                                'username': self.username,
                                'password': self.password,
                            }
                        },
                        connections = {
                            'cli': {
                                'protocol': 'ssh',
                                'ip': self.hostname,
                                'port': 22,
                                'arguements': {
                                    'connection_timeout': 360
                                }
                            }
                        })
            testbed_device.testbed = first_testbed
            new_testbed = testbed.load(first_testbed)
        except Exception as e:
            logging.exception(e)
        return(new_testbed)

    def capture_state(self):
        # ---------------------------------------
        # Loop over devices
        # ---------------------------------------
        for device in self.connect_device():
            device.connect(log_stdout=False)
        # Parse or Learn based on command
            if 'show' in self.command:
                try:
                    command_output = device.parse(self.command)
                except:
                    command_output = f"Cannot Parse { self.command }"
            elif self.command == "config":
                try:
                    command_output = device.learn(self.command)
                except:
                    command_output = "Cannot Learn Config"
            elif self.command == "platform":
                try:
                    command_output = device.learn(self.command).to_dict()
                except:
                    command_output = "Cannot Learn Platform"
            else:
                try:
                    command_output = device.learn(self.command).info
                except:
                    command_output = f"Cannot Learn { self.command }"
            return(command_output)

@click.command()
@click.option('--hostname', prompt='Hostname', help='Hostname of device - must match the device', required=True)
@click.option('--os', prompt='OS', type=click.Choice(['ios', 'iosxe', 'iosxr', 'nxos'], case_sensitive=True), help='OS of device - must match the device', required=True)
@click.option('--username', prompt='Username', help='Username', required=True)
@click.option('--password', prompt=True, hide_input=True, help="User Password", required=True)
@click.option('--command', prompt='Command', help='A valid pyATS Learn Function (i.e. ospf) or valid CLI Show Command (i.e. "show ip interface brief")', required=True)
def cli(hostname, os, username, password, command):
    invoke_class = GetJson(hostname, os, username, password, command)
    invoke_class.print_json()

if __name__ == "__main__":
    cli()